       >>SOURCE FORMAT IS FIXED
      ******************************************************************
      * Author: Jay Moseley
      * Date: July 23, 2020
      * Purpose: Create Hercules Emulated Tape (.het) file, containing 
      *          IBM standard tape image with the contents of one or 
      *          more files from the host Operating System as datasets 
      *          on the tape image. 
      *          
      *          This source file contains three programs:
      *            maketapeScreen - acquires parameters from user
      *            createHetImage - builds het image
      *            AtoEtoA - ASCII to EBCDIC / EBCDIC to ASCII translate
      *
      * Tectonics: cobc -x maketape.cbl
      *----------------------------------------------------------------*
      *                                                                *
      * 06 April 2021 - GnuCOBOL 3.1.2.0 introduced a fix for data     *
      *                 items defined with ANY LENGTH, which rendered  *
      *                 compile errors to coding methodolgy I had been *
      *                 using. Revision made to this code [subprogram  *
      *                 AtoEtoA] to comply with the fix implemented.   *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. maketapeScreen.
       AUTHOR.
       DATE-WRITTEN. May 17, 2020.
       DATE-COMPILED.

       REMARKS. This program is placed in the public domain and may be
                freely used and incorporated into derived works so
                long as attribution to the original authorship remains 
                in any distributed copies of the COBOL source.

       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       REPOSITORY.
           FUNCTION ALL INTRINSIC.
       SPECIAL-NAMES.
           CURSOR IS LLCC.

       INPUT-OUTPUT SECTION.
       FILE-CONTROL.

           SELECT META-FILE
               ASSIGN TO WS-INPUT-FILE
               ORGANIZATION IS LINE SEQUENTIAL.

       I-O-CONTROL.

       DATA DIVISION.
       FILE SECTION.

       FD  META-FILE
           DATA RECORD IS META-RECORD.
       01  META-RECORD                 PIC X(64).

       WORKING-STORAGE SECTION.

       01  LLCC                        PIC 9(04).

       01  SCREEN-FIELDS.
           02  SF-INPUT-FILE           PIC X(64).
           02  SF-OUTPUT-FILE          PIC X(64).
           02  SF-VOLSER               PIC X(06).
           02  SF-DATASET              PIC X(17).
           02  SF-LRECL                PIC X(04).
           02  SF-LRECL-9              REDEFINES SF-LRECL
                                       PIC 9(04).
           02  SF-BLKSIZE              PIC X(04).
           02  SF-BLKSIZE-9            REDEFINES SF-BLKSIZE
                                       PIC 9(04).
           02  SF-GLOB-FILES           PIC X(01).
           02  SF-MESSAGE              PIC X(78).

       01  WS-FILE-INFO                USAGE COMP.
           02  WS-FILE-SIZE            PIC 9(18).
           02  WS-FILE-MOD-DMY         PIC 9(08).
           02  WS-FILE-MOD-HMS         PIC 9(06).
           02  FILLER                  PIC 9(02).

       01  FIELD-ERROR-SWITCH          PIC X.
           88  FIELD-ERROR-FOUND       VALUE 'Y'.
           88  FIELD-ERROR-NOT-FOUND   VALUE 'N'.

       01  WS-INPUT-FILE               PIC X(64).
       01  WS-META-FILE-COUNT          PIC 9(04).
       01  WS-META-FILE-ID             PIC X(64).
       01  WS-BLOCK-FACTOR             PIC 9(04).
       01  WS-EDIT-LRECL               PIC ZZZ9.
       01  WS-EDIT-BLOCK-FACTOR        PIC ZZZ9.
       01  WS-UNIQUE                   PIC X(6).

       COPY 'screenio.cpy'.      

       SCREEN SECTION.

       01  ENTRY-SCREEN.
           02  LINE 01 COLUMN 01
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               ERASE EOS.
           02  LINE 01 COLUMN 19 
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(43) 
               VALUE 'Load Text File(s) to ' &
               'Hercules Emulated Tape'.

           02  LINE 03 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(17)
               VALUE 'Text input file: '.
           02  ES-INPUT-FILE
               LINE 03 COLUMN 23
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK
               PIC X(64)
               USING SF-INPUT-FILE.

           02  LINE 04 COLUMN 23
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(48)
               VALUE '(If first character = @, ' &
               'input file is metafile)'.

           02  LINE 06 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(24)
               VALUE 'Tape image output file: '.
           02  ES-OUTPUT-FILE
               LINE 06 COLUMN 30
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK
               PIC X(64)
               USING SF-OUTPUT-FILE.

           02  LINE 08 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(27)
               VALUE 'Tape Volume Serial Number: '.
           02  ES-VOLSER
               LINE 08 COLUMN 33
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK 
               PIC X(06)
               USING SF-VOLSER.

           02  LINE 10 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(19)
               VALUE 'Tape Dataset Name: '.
           02  ES-DATASET
               LINE 10 COLUMN 25
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK 
               PIC X(17)
               USING SF-DATASET.

           02  LINE 12 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(15)
               VALUE 'Record length: '.
           02  ES-LRECL
               LINE 12 COLUMN 21
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK 
               PIC X(4)
               USING SF-LRECL.

           02  LINE 14 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(14)
               VALUE 'Block Length: '.
           02  ES-BLKSIZE
               LINE 14 COLUMN 20
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK 
               PIC X(4)
               USING SF-BLKSIZE.

           02  LINE 16 COLUMN 06
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               PIC X(50)
               VALUE 'Combine all input files into ' &
                     'single output (Y/N): '.
           02  ES-UNIQUE
               LINE 16 COLUMN 56
               BACKGROUND-COLOR IS COB-COLOR-WHITE
               FOREGROUND-COLOR IS COB-COLOR-BLACK 
               PIC X(1)
               USING SF-GLOB-FILES.

           02  LINE 18 COLUMN 01
               BACKGROUND-COLOR IS COB-COLOR-BLACK
               FOREGROUND-COLOR IS COB-COLOR-WHITE HIGHLIGHT
               VALUE 'Complete fields above and ' &
               'press Enter to create tape. ' &
               'Press F3 to terminate.'.

       01  CLEAR-SCREEN.
           02  LINE 01 COLUMN 01
               ERASE EOS
               VALUE ' '.

       PROCEDURE DIVISION.

       HOUSEKEEPING.

           SET ENVIRONMENT 'COB_EXIT_WAIT' to 'N'.
           DISPLAY ' ' LINE 1 POSITION 1.
           INITIALIZE SCREEN-FIELDS.
           MOVE 'Y' TO SF-GLOB-FILES.

       SHOW-ENTRY-SCREEN.
           MOVE 0101 TO LLCC.
           DISPLAY ENTRY-SCREEN.

           PERFORM WITH TEST AFTER
                   UNTIL FIELD-ERROR-NOT-FOUND

             PERFORM VERIFY-FIELDS
                THRU VERIFY-FIELDS-EXIT

             IF COB-CRT-STATUS = COB-SCR-F3
               EXIT PERFORM
             END-IF

           END-PERFORM.

           IF COB-CRT-STATUS NOT = COB-SCR-F3
             IF FIELD-ERROR-NOT-FOUND
               CALL 'createHetImage' USING SF-INPUT-FILE
                                           SF-OUTPUT-FILE
                                           UPPER-CASE(SF-VOLSER)
                                           UPPER-CASE(SF-DATASET)
                                           SF-LRECL
                                           SF-BLKSIZE
                                           SF-GLOB-FILES
               END-CALL
             END-IF
           END-IF.

           DISPLAY CLEAR-SCREEN.
           GOBACK.

       VERIFY-FIELDS.

           ACCEPT ENTRY-SCREEN 

           IF COB-CRT-STATUS = COB-SCR-F3
             GO TO VERIFY-FIELDS-EXIT
           END-IF

           SET FIELD-ERROR-NOT-FOUND TO TRUE.
           MOVE SPACES TO SF-MESSAGE.
           DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01.

      *> Text input file must be entered and must exist

           IF SF-INPUT-FILE = SPACES
             MOVE 'Text input file must be entered'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
             DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 0323 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.

           IF SF-INPUT-FILE (1:1) = '@'
             MOVE SF-INPUT-FILE (2:) TO WS-INPUT-FILE
           ELSE
             MOVE SF-INPUT-FILE TO WS-INPUT-FILE
           END-IF.

           CALL 'CBL_CHECK_FILE_EXIST'
             USING WS-INPUT-FILE, WS-FILE-INFO.
           IF RETURN-CODE = 35
             MOVE 'Text input file is not found'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
             DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
             END-DISPLAY
             MOVE 0323 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.

      *> Special tests for meta file

           IF SF-INPUT-FILE (1:1) = '@'
             PERFORM TEST-METAFILE
                THRU TEST-METAFILE-EXIT
             IF FIELD-ERROR-FOUND
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
                 WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
               MOVE 0323 TO LLCC
               GO TO VERIFY-FIELDS-EXIT
             END-IF
           END-IF.

      *> Tape image file must be entered and NOT exist

           IF SF-OUTPUT-FILE = SPACES
             MOVE 'Tape image output file must be entered'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 0630 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.

           CALL 'CBL_CHECK_FILE_EXIST'
             USING SF-OUTPUT-FILE, WS-FILE-INFO.
           IF RETURN-CODE NOT = 35
             MOVE 'Tape image output file already exists'        
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 0630 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.

      *> Tape Volume Serial # must have some entry

           IF SF-VOLSER = SPACES
             MOVE 'Tape Volume Serial Number may not be blank'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 0833 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.

      *> Dataset name must have some entry

           IF SF-DATASET = SPACES
             MOVE 'Tape Dataset Name may not be blank'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 1025 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.

      *> Record length must be entered
      *>               and numeric
      *>               and not zero

           IF (SF-LRECL = SPACES OR ZEROS)
           OR (TEST-NUMVAL(SF-LRECL) NOT = ZERO)
             MOVE 'Record length must be entered, numeric, and > zero'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 1221 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.
         MOVE NUMVAL(SF-LRECL) TO SF-LRECL-9.

      *> Block length must be entered
      *>               and numeric
      *>               and not zero

           IF (SF-BLKSIZE = SPACES OR ZEROS)
           OR (TEST-NUMVAL(SF-BLKSIZE) NOT = ZERO)
             MOVE 'Block length must be entered, numeric, > zero'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 1420 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.
           MOVE NUMVAL(SF-BLKSIZE) TO SF-BLKSIZE-9.

      *> Block length must be multiple of record length

           IF MOD(SF-BLKSIZE, SF-LRECL) NOT = ZERO
             MOVE 'Block length must be multiple of record length'
               TO SF-MESSAGE
             SET FIELD-ERROR-FOUND TO TRUE
               DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
               WITH BACKGROUND-COLOR COB-COLOR-RED
                    FOREGROUND-COLOR COB-COLOR-WHITE
                    BELL
               END-DISPLAY
             MOVE 1420 TO LLCC
             GO TO VERIFY-FIELDS-EXIT
           END-IF.
           COMPUTE WS-BLOCK-FACTOR = 
                   SF-BLKSIZE-9 / SF-LRECL-9.

      *> Unique must be Y or N

           MOVE UPPER-CASE(SF-GLOB-FILES) TO SF-GLOB-FILES.
           EVALUATE SF-GLOB-FILES
             WHEN 'Y'
               MOVE SPACES TO WS-UNIQUE
             WHEN 'N'
               MOVE 'UNIQUE' TO WS-UNIQUE
             WHEN OTHER
               MOVE 'Option must be either Y or N'
                 TO SF-MESSAGE
               SET FIELD-ERROR-FOUND TO TRUE
                 DISPLAY SF-MESSAGE AT LINE 24 COLUMN 01
                 WITH BACKGROUND-COLOR COB-COLOR-RED
                      FOREGROUND-COLOR COB-COLOR-WHITE
                      BELL
                 END-DISPLAY
               MOVE 1656 TO LLCC
               GO TO VERIFY-FIELDS-EXIT
           END-EVALUATE.
 
       VERIFY-FIELDS-EXIT.
           EXIT.

       TEST-METAFILE.

      *> Meta file may contain no more than 50 entries
      *> Each entry must be the name of a file that exists

           OPEN INPUT META-FILE.
           MOVE ZERO TO WS-META-FILE-COUNT.

           PERFORM FOREVER
             READ META-FILE
               AT END
                 EXIT PERFORM
               NOT AT END
                 ADD 1 TO WS-META-FILE-COUNT
                 IF WS-META-FILE-COUNT > 50
                   SET FIELD-ERROR-FOUND TO TRUE
                   MOVE 'Meta file contains more than ' &
                        '50 files' TO SF-MESSAGE
                   EXIT PERFORM
                 END-IF
                 CALL 'CBL_CHECK_FILE_EXIST'
                   USING META-RECORD, WS-FILE-INFO
                 END-CALL
                 IF RETURN-CODE = 35
                   SET FIELD-ERROR-FOUND TO TRUE
                   MOVE CONCATENATE('Meta file #',
                                    WS-META-FILE-COUNT,
                                    ' is not found')
                     TO SF-MESSAGE
                   EXIT PERFORM
                 END-IF
             END-READ
           END-PERFORM.

           CLOSE META-FILE.

       TEST-METAFILE-EXIT.
           EXIT.

       END PROGRAM maketapeScreen.

      ******************************************************************
      * Author: Jay Moseley
      * Date: July 21, 2020
      * Purpose: Create Hercules emulated tape (.het) file containing
      *          IBM standard tape image with MVS standard labels; the
      *          tape image will contain the contents of one or more 
      *          files from the host Operating System as datasets on the
      *          tape image. 
      * Tectonics: embedded in maketape.cbl source file
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. createHetImage.
       AUTHOR.
       DATE-WRITTEN. July 21, 2020.
       DATE-COMPILED.

       REMARKS. This program is placed in the public domain and may be
                freely used and incorporated into derived works so
                long as attribution to the original authorship remains 
                in any distributed copies of the COBOL source.

       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       REPOSITORY.
           FUNCTION ALL INTRINSIC.

       INPUT-OUTPUT SECTION.
       FILE-CONTROL.

           SELECT INPUT-FILE
               ASSIGN TO WS-INPUT-FILE
               ORGANIZATION IS LINE SEQUENTIAL.

       I-O-CONTROL.

       DATA DIVISION.
       FILE SECTION.

       FD  INPUT-FILE
           DATA RECORD IS INPUT-RECORD.
       01  INPUT-RECORD                PIC X(32768).

       WORKING-STORAGE SECTION.

       *> Fields related to input file(s)
       01  WS-INPUT-FILE               PIC X(64).
       01  WS-INPUT-END-OF-FILE        PIC X VALUE 'N'.
           88  END-OF-INPUT-FILE             VALUE 'Y'.
           88  NOT-END-OF-INPUT-FILE         VALUE 'N'.
       01  WS-META-FILES.
           02  WS-META-FILE-COUNT      PIC S9(4) COMP.
           02  WS-META-FILE-NAME       OCCURS 50 TIMES
                                       PIC X(64).
       01  WS-CURRENT-FILE             PIC S9(4) COMP VALUE +1.

       *> Fields related to output file
       01  WS-OUTPUT-FILE              PIC X(64).
       01  WS-DATASET-NAME             PIC X(34).
       01  WS-BLOCKS-WRITTEN-EDIT      PIC ZZZ,ZZ9.

       *> MVS label information
       01  MVS-LABEL-TYPE              PIC X.
       01  MVS-LABEL-SEQUENCE          PIC 9.
       01  MVS-FILE-NUMBER             PIC 9(4) VALUE 1.
       01  MVS-FILE-NAME               PIC X(17).
       01  MVS-JULIAN-DATE             PIC 9(5).
       01  MVS-RECFM                   PIC X.
       01  MVS-LRECL                   PIC 9(5).
       01  MVS-BLKSIZE                 PIC 9(5).
       01  MVS-BLOCK-COUNT             PIC 9(6).

       *> Buffer for building output record
       01  OUTPUT-BUFFER-PTR           PIC S9(5) COMP VALUE +1.
       01  OUTPUT-BUFFER-SIZE          PIC S9(5) COMP VALUE +0.
       01  OUTPUT-BUFFER               PIC X(32768) VALUE SPACES.

       *> Parameters for CBL_ file handling routines.
       01  CBL-PARMS.
           02  CBL-HANDLE              PIC X(4) COMP-X.
           02  CBL-OFFSET              PIC X(8) COMP-X VALUE 0.
           02  CBL-NBYTES              PIC X(4) COMP-X VALUE 0.
           02  CBL-BUFFER              PIC X(32768) VALUE SPACES.

       *> Header record 
       01  HEADER-RECORD.
           02  HDR-CURRENT-LRECL       PIC S9(4) COMP-5 VALUE 0.
           02  HDR-PREVIOUS-LRECL      PIC S9(4) COMP-5 VALUE 0.
           02  HDR-FLAGS               PIC S9(4) COMP-5 VALUE 0.

       LINKAGE SECTION.

       01  LS-INPUT-FILE               PIC X(64).
       01  LS-OUTPUT-FILE              PIC X(64).
       01  LS-VOLSER                   PIC X(06).
       01  LS-DATASET                  PIC X(17).
       01  LS-LRECL                    PIC X(04).
       01  LS-BLKSIZE                  PIC X(04).
       01  LS-GLOB-FILES               PIC X(01).
           88  SINGLE-OUTPUT-DATASET             VALUE 'Y'.
           88  MULTIPLE-OUTPUT-DATASETS          VALUE 'N'.


       PROCEDURE DIVISION USING LS-INPUT-FILE, LS-OUTPUT-FILE,
                                LS-VOLSER, LS-DATASET, LS-LRECL,
                                LS-BLKSIZE, LS-GLOB-FILES.

       HOUSEKEEPING.

      *> Julian date is tape creation date in standard labels
           ACCEPT MVS-JULIAN-DATE FROM DAY.

      ******************************************************************
      * Test input file spec to see if it is meta file. If meta file   *
      * has been specified, load contents to array for processing,     *
      * otherwise place specified file name into first slot in array.  *
      ******************************************************************
           IF LS-INPUT-FILE (1:1) = '@'
             MOVE LS-INPUT-FILE (2:) TO WS-INPUT-FILE
             INITIALIZE WS-META-FILES
             OPEN INPUT INPUT-FILE
             PERFORM FOREVER
               READ INPUT-FILE
                 AT END
                   EXIT PERFORM
                 NOT AT END
                   ADD 1 TO WS-META-FILE-COUNT
                   MOVE INPUT-RECORD (1:64) 
                     TO WS-META-FILE-NAME (WS-META-FILE-COUNT),
               END-READ
             END-PERFORM
             CLOSE INPUT-FILE
           ELSE
             MOVE 1 TO WS-META-FILE-COUNT
             MOVE LS-INPUT-FILE TO WS-META-FILE-NAME (1),
                                     WS-INPUT-FILE
           END-IF.

           MOVE LS-LRECL TO MVS-LRECL.
           MOVE LS-BLKSIZE TO MVS-BLKSIZE.

       BUILD-AWS-IMAGE.

      * ************************************************************* *
      * Process file with low-level I/O routines                      *
      * ************************************************************* *
           
           MOVE LS-OUTPUT-FILE TO WS-OUTPUT-FILE.
           CALL 'CBL_OPEN_FILE' USING WS-OUTPUT-FILE, 2, 0, 0, 
                                      CBL-HANDLE.
           IF RETURN-CODE NOT = ZERO
             DISPLAY 'Non-zero return code on CBL_OPEN_FILE ('
                     RETURN-CODE ')'
             DISPLAY 'File name: ' TRIM(WS-OUTPUT-FILE)
             END-DISPLAY
             STOP RUN 99
           END-IF.

      * ************************************************************* *
      * Write VOL1 label                                              *
      * ************************************************************* *
           MOVE 'VOL1' TO OUTPUT-BUFFER.
           MOVE LS-VOLSER TO OUTPUT-BUFFER (5:6).
           MOVE 80 TO OUTPUT-BUFFER-SIZE.
           PERFORM WRITE-BUFFER.

      * ************************************************************* *
      * If merging all input files into single output dataset, write  *
      * standard header label set (HDR1/HDR2)                         *
      * ************************************************************* *
           IF SINGLE-OUTPUT-DATASET
             MOVE LS-DATASET TO MVS-FILE-NAME
             MOVE 'H' TO MVS-LABEL-TYPE
             MOVE 1 TO MVS-LABEL-SEQUENCE
             PERFORM WRITE-STANDARD-LABEL
             MOVE 2 TO MVS-LABEL-SEQUENCE
             PERFORM WRITE-STANDARD-LABEL
             PERFORM WRITE-TAPE-MARK
           END-IF

      * ************************************************************* *
      * Process until all input files have been included              *
      * ************************************************************* *

           PERFORM UNTIL WS-CURRENT-FILE > WS-META-FILE-COUNT

             MOVE WS-META-FILE-NAME (WS-CURRENT-FILE)
               TO WS-INPUT-FILE
             DISPLAY 'Processing input from: ' 
                     TRIM(WS-INPUT-FILE)
             END-DISPLAY
             SET NOT-END-OF-INPUT-FILE TO TRUE
             OPEN INPUT INPUT-FILE

      * ************************************************************* *
      * If writing multiple input files to multiple datasets on tape, *
      * write header label set before data on tape.                   *
      * ************************************************************* *
             IF MULTIPLE-OUTPUT-DATASETS
               MOVE LS-DATASET TO MVS-FILE-NAME
               MOVE 'H' TO MVS-LABEL-TYPE
               MOVE 1 TO MVS-LABEL-SEQUENCE
               PERFORM WRITE-STANDARD-LABEL
               MOVE 2 TO MVS-LABEL-SEQUENCE
               PERFORM WRITE-STANDARD-LABEL
               PERFORM WRITE-TAPE-MARK
               MOVE +0 TO MVS-BLOCK-COUNT
               MOVE +0 TO OUTPUT-BUFFER-SIZE
               MOVE +1 TO OUTPUT-BUFFER-PTR
               MOVE SPACES TO OUTPUT-BUFFER
             END-IF

             PERFORM FOREVER

               READ INPUT-FILE
                 AT END
                   EXIT PERFORM
                 NOT AT END
                   PERFORM PUT-RECORD-IN-BUFFER
               END-READ

             END-PERFORM

      * ************************************************************* *
      * If writing multiple input files to multiple datasets on tape, *
      * write trailer label set after data on tape.                   *
      * ************************************************************* *
             IF MULTIPLE-OUTPUT-DATASETS
      *> If partial block has been created in buffer, write it to file
               IF OUTPUT-BUFFER-SIZE > 0
                 PERFORM WRITE-BUFFER
                 ADD +1 TO MVS-BLOCK-COUNT
               END-IF
               PERFORM WRITE-TAPE-MARK
               MOVE 'E' TO MVS-LABEL-TYPE
               MOVE 1 TO MVS-LABEL-SEQUENCE
               PERFORM WRITE-STANDARD-LABEL
               MOVE 2 TO MVS-LABEL-SEQUENCE
               PERFORM WRITE-STANDARD-LABEL
               PERFORM WRITE-TAPE-MARK
               MOVE MVS-BLOCK-COUNT TO WS-BLOCKS-WRITTEN-EDIT
               DISPLAY 'Wrote '
                       TRIM(WS-BLOCKS-WRITTEN-EDIT)
                       ' blocks to AWSTAPE file: '
                       TRIM(WS-OUTPUT-FILE)
                       ' (Seq #' MVS-FILE-NUMBER
                       ' Dataset: '
                       TRIM(MVS-FILE-NAME) ')'
               END-DISPLAY
               ADD 1 TO MVS-FILE-NUMBER
             END-IF
 
             CLOSE INPUT-FILE
             ADD +1 TO WS-CURRENT-FILE

           END-PERFORM.

      * ************************************************************* *
      * If merging all input files into single output dataset, write  *
      * standard trailer label set (EOF1/EOF2)                        *
      * ************************************************************* *
           IF SINGLE-OUTPUT-DATASET
      *> If partial block has been created in buffer, write it to file
             IF OUTPUT-BUFFER-SIZE > 0
               PERFORM WRITE-BUFFER
               ADD +1 TO MVS-BLOCK-COUNT
             END-IF
             PERFORM WRITE-TAPE-MARK
             MOVE 'E' TO MVS-LABEL-TYPE
             MOVE 1 TO MVS-LABEL-SEQUENCE
             PERFORM WRITE-STANDARD-LABEL
             MOVE 2 TO MVS-LABEL-SEQUENCE
             PERFORM WRITE-STANDARD-LABEL
             PERFORM WRITE-TAPE-MARK
             MOVE MVS-BLOCK-COUNT TO WS-BLOCKS-WRITTEN-EDIT
             DISPLAY 'Wrote '
                     TRIM(WS-BLOCKS-WRITTEN-EDIT)
                     ' blocks to AWSTAPE file: '
                     TRIM(WS-OUTPUT-FILE)
                     ' (Seq #' MVS-FILE-NUMBER
                     ' Dataset: '
                     TRIM(MVS-FILE-NAME) ')'
             END-DISPLAY

           END-IF.

      * ************************************************************* *
      * Double tape marks indicate end-of-tape, so write a final tape *
      * mark on output image.                                         *
      * ************************************************************* *
           PERFORM WRITE-TAPE-MARK.

           CALL 'CBL_CLOSE_FILE' USING CBL-HANDLE.
           GOBACK.
      * - - - - - - - - - - - - - - - - - - - - - - - Program Exit Point

      * ************************************************************* *
      * Store input record in output buffer. If buffer has reached    *
      * the specified block size, write the buffer to the tape image. *
      * ************************************************************* *
       PUT-RECORD-IN-BUFFER.

           MOVE INPUT-RECORD (1:MVS-LRECL) 
             TO OUTPUT-BUFFER (OUTPUT-BUFFER-PTR:MVS-LRECL).
           ADD MVS-LRECL TO OUTPUT-BUFFER-SIZE,
                            OUTPUT-BUFFER-PTR.
           IF OUTPUT-BUFFER-PTR > MVS-BLKSIZE
             PERFORM WRITE-BUFFER
             ADD +1 TO MVS-BLOCK-COUNT
             MOVE +0 TO OUTPUT-BUFFER-SIZE
             MOVE +1 TO OUTPUT-BUFFER-PTR
             MOVE SPACES TO OUTPUT-BUFFER
           END-IF.
      * - - - - - - - - - - - - - - - - - - - - - - - Perform Exit Point

      * ************************************************************* *
      * Write a tape mark to the tape image. A tape mark is defined   *
      * as a header with no data following it.                        *
      * ************************************************************* *
       WRITE-TAPE-MARK.
 
           MOVE 0 TO OUTPUT-BUFFER-SIZE.
           PERFORM WRITE-BUFFER.
      * - - - - - - - - - - - - - - - - - - - - - - - Perform Exit Point

      * ************************************************************* *
      * Create and write standard label set (HDR1/HDR2 or EOF1/EOF2)  *
      * ************************************************************* *
       WRITE-STANDARD-LABEL.

      *> Prior to HDR1, create proper MVS file name for dataset. If
      *> all input files are merged into a single output, use the
      *> name specified by user, otherwise append sequence of input
      *> file to the name specified by user.
           IF MVS-LABEL-TYPE = 'H'
          AND MVS-LABEL-SEQUENCE = 1
          AND MULTIPLE-OUTPUT-DATASETS
             MOVE CONCATENATE(TRIM(MVS-FILE-NAME),
                              '.F',
                              MVS-FILE-NUMBER)
               TO WS-DATASET-NAME
             IF LENGTH(TRIM(WS-DATASET-NAME)) > 17
               MOVE REVERSE(REVERSE(TRIM(WS-DATASET-NAME)) (1:17))
                 TO MVS-FILE-NAME
             ELSE
               MOVE WS-DATASET-NAME (1:17) TO MVS-FILE-NAME
             END-IF
           END-IF

           EVALUATE TRUE 
             WHEN MVS-LABEL-TYPE = 'H'
               MOVE 'HDR' TO OUTPUT-BUFFER 
               MOVE MVS-LABEL-SEQUENCE TO OUTPUT-BUFFER (4:1)

             WHEN MVS-LABEL-TYPE = 'E'
               MOVE 'EOF' TO OUTPUT-BUFFER
               MOVE MVS-LABEL-SEQUENCE TO OUTPUT-BUFFER (4:1)
           END-EVALUATE.

           EVALUATE TRUE
             WHEN MVS-LABEL-SEQUENCE = 1
               MOVE MVS-FILE-NAME TO OUTPUT-BUFFER (5:17)
               MOVE LS-VOLSER TO OUTPUT-BUFFER (22:6)
               MOVE '0001' TO OUTPUT-BUFFER (28:4)
               IF MULTIPLE-OUTPUT-DATASETS
                 MOVE MVS-FILE-NUMBER TO OUTPUT-BUFFER (32:4)
               ELSE
                 MOVE '0001' TO OUTPUT-BUFFER (32:4)
               END-IF
               MOVE '0' TO OUTPUT-BUFFER (42:1)
               MOVE MVS-JULIAN-DATE TO OUTPUT-BUFFER (43:5)
               MOVE '099365' TO OUTPUT-BUFFER (48:6)
               MOVE '0' TO OUTPUT-BUFFER (54:1)
               IF MVS-LABEL-TYPE = 'H'
                   MOVE '000000' TO OUTPUT-BUFFER (55:6)
               ELSE
                   MOVE MVS-BLOCK-COUNT TO OUTPUT-BUFFER (55:6)
               END-IF
               
             WHEN MVS-LABEL-SEQUENCE = 2
               MOVE 'F' TO OUTPUT-BUFFER (5:1)
               MOVE MVS-BLKSIZE TO OUTPUT-BUFFER (6:5)
               MOVE MVS-LRECL TO OUTPUT-BUFFER (11:5)
               MOVE '30' TO OUTPUT-BUFFER (16:2)
               MOVE 'EXTERNAL/MAKETAPE' TO OUTPUT-BUFFER (18:17)
               IF MVS-BLKSIZE > MVS-LRECL
                 MOVE 'B' TO OUTPUT-BUFFER (39:1)
               END-IF
           END-EVALUATE.

           MOVE 80 TO OUTPUT-BUFFER-SIZE.
           PERFORM WRITE-BUFFER.
      * - - - - - - - - - - - - - - - - - - - - - - - Perform Exit Point

      * ************************************************************* *
      * Write buffer to output file. Each virtual tape block has a 6  *
      * character header preceeding it. All blocks except for TAPE    *
      * MARK will have data following the header.                     *
      * ************************************************************* *
       WRITE-BUFFER.

           MOVE OUTPUT-BUFFER-SIZE TO HDR-CURRENT-LRECL.
           IF HDR-CURRENT-LRECL = 0
             MOVE 64 TO HDR-FLAGS
           ELSE
             MOVE 160 TO HDR-FLAGS
           END-IF.
           MOVE HEADER-RECORD TO CBL-BUFFER.
           MOVE 6 TO CBL-NBYTES.
           PERFORM CALL-CBL-WRITE.
           IF HDR-CURRENT-LRECL > 0
             CALL 'TOEBCDIC' USING OUTPUT-BUFFER (1:OUTPUT-BUFFER-SIZE)
             MOVE OUTPUT-BUFFER TO CBL-BUFFER
             MOVE OUTPUT-BUFFER-SIZE TO CBL-NBYTES
             PERFORM CALL-CBL-WRITE
           END-IF.
      * - - - - - - - - - - - - - - - - - - - - - - - Perform Exit Point

      * ************************************************************* *
      * Call low level routine to write to binary file. If successful *
      * write occurs, increment output pointer by number of bytes     *
      * written.                                                      *
      * ************************************************************* *
       CALL-CBL-WRITE.

           CALL 'CBL_WRITE_FILE' USING CBL-HANDLE, CBL-OFFSET,
                                       CBL-NBYTES, 0, CBL-BUFFER.
           EVALUATE TRUE
             WHEN RETURN-CODE = 0
               ADD CBL-NBYTES TO CBL-OFFSET
               MOVE HDR-CURRENT-LRECL TO HDR-PREVIOUS-LRECL
             WHEN OTHER
               DISPLAY 'Non-zero return code on CBL_WRITE_FILE ('
                       RETURN-CODE ')'
               END-DISPLAY
               STOP RUN 99
           END-EVALUATE.
      * - - - - - - - - - - - - - - - - - - - - - - - Perform Exit Point

       END PROGRAM createHetImage.

      ******************************************************************
      * Author: Jay Moseley
      * Date: July 19, 2020
      * Purpose: Convert passed record from EBCDIC to ASCII.
      * Tectonics: cobc -m AtoEtoA
      *----------------------------------------------------------------*
      *                                                                *
      * 06 April 2021 - GnuCOBOL 3.1.2.0 introduced a fix for data     *
      *                 items defined with ANY LENGTH, which rendered  *
      *                 compile errors to coding methodolgy I had been *
      *                 using. Revision made to this code to comply    *
      *                 with the fix implemented.                      *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. AtoEtoA.

       ENVIRONMENT DIVISION.

       CONFIGURATION SECTION.
       REPOSITORY.
           FUNCTION ALL INTRINSIC.

       DATA DIVISION.

       WORKING-STORAGE SECTION.

       *> EBCDIC to ASCII:
       01  EBCDIC-DATA.
           02  PIC X(16) VALUE X'000102030405060708090A0B0C0D0E0F'.      000-015
           02  PIC X(16) VALUE X'101112131415161718191A1B1C1D1E1F'.      016-031
           02  PIC X(16) VALUE X'202122232425262728292A2B2C2D2E2F'.      032-047
           02  PIC X(16) VALUE X'303132333435363738393A3B3C3D3E3F'.      048-063
           02  PIC X(16) VALUE X'404142434445464748494A4B4C4D4E4F'.      064-079
           02  PIC X(16) VALUE X'505152535455565758595A5B5C5D5E5F'.      080-095
           02  PIC X(16) VALUE X'606162636465666768696A6B6C6D6E6F'.      096-111
           02  PIC X(16) VALUE X'707172737475767778797A7B7C7D7E7F'.      112-127
           02  PIC X(16) VALUE X'808182838485868788898A8B8C8D8E8F'.      128-143
           02  PIC X(16) VALUE X'909192939495969798999A9B9C9D9E9F'.      144-159
           02  PIC X(16) VALUE X'A0A1A2A3A4A5A6A7A8A9AAABACADAEAF'.      160-175
           02  PIC X(16) VALUE X'B0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF'.      176-191
           02  PIC X(16) VALUE X'C0C1C2C3C4C5C6C7C8C9CACBCCCDCECF'.      192-207
           02  PIC X(16) VALUE X'D0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF'.      208-223
           02  PIC X(16) VALUE X'E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEF'.      224-239
           02  PIC X(16) VALUE X'F0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF'.      240-255
       01  EBCDIC-ENCODING REDEFINES EBCDIC-DATA PIC X(256).

       *> ASCII to EBCDIC:
       01  ASCII-DATA.
           02  PIC X(16) VALUE X'000102039C09867F978D8E0B0C0D0E0F'.      000-015
           02  PIC X(16) VALUE X'101112139D8508871819928F1C1D1E1F'.      016-031
           02  PIC X(16) VALUE X'80818283840A171B88898A8B8C050607'.      032-047
           02  PIC X(16) VALUE X'909116939495960498999A9B14159E1A'.      048-063
           02  PIC X(16) VALUE X'20A0E2E4E0E1E3E5E7F1A22E3C282B7C'.      064-079
           02  PIC X(16) VALUE X'26E9EAEBE8EDEEEFECDF21242A293BAC'.      080-095
           02  PIC X(16) VALUE X'2D2FC2C4C0C1C3C5C7D1A62C255F3E3F'.      096-111
           02  PIC X(16) VALUE X'F8C9CACBC8CDCECFCC603A2340273D22'.      112-127
           02  PIC X(16) VALUE X'D8616263646566676869ABBBF0FDFEB1'.      128-143
           02  PIC X(16) VALUE X'B06A6B6C6D6E6F707172AABAE6B8C6A4'.      144-159
           02  PIC X(16) VALUE X'B57E737475767778797AA1BFD0DDDEAE'.      160-175
           02  PIC X(16) VALUE X'5EA3A5B7A9A7B6BCBDBE5B5DAFA8B4D7'.      176-191
           02  PIC X(16) VALUE X'7B414243444546474849ADF4F6F2F3F5'.      192-207
           02  PIC X(16) VALUE X'7D4A4B4C4D4E4F505152B9FBFCF9FAFF'.      208-223
           02  PIC X(16) VALUE X'5CF7535455565758595AB2D4D6D2D3D5'.      224-239
           02  PIC X(16) VALUE X'30313233343536373839B3DBDCD9DA9F'.      240-255
       01  ASCII-ENCODING REDEFINES ASCII-DATA PIC X(256).

       LINKAGE SECTION.

       *> Buffer from caller, containing string:
       01  PASSED-BUFFER               PIC X ANY LENGTH.

       PROCEDURE DIVISION USING PASSED-BUFFER.

       MAIN-PROCEDURE.

       ENTRY 'TOASCII' USING PASSED-BUFFER.

           TRANSFORM PASSED-BUFFER
             FROM EBCDIC-ENCODING TO ASCII-ENCODING.

           GOBACK.    

       ENTRY 'TOEBCDIC' USING PASSED-BUFFER.

           TRANSFORM PASSED-BUFFER
             FROM ASCII-ENCODING TO EBCDIC-ENCODING.

           GOBACK.    

       END PROGRAM AtoEtoA.
